package fantasy;
/*
Copyright (C) 2014 Johan Ceuppens 
Also Copyrights
From the Book : "Teach yourself Internet Game Programming with Java in 21 days"

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.ImageIcon;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import java.util.*;

class SpriteVector extends Vector 
{
protected Background background;

public SpriteVector(Background back) { 
	super(50, 10); 
	background = back;
}

public Background getBackground() { 
	return background;
}

public void setBackground(Background back) { 
	background = back;
}

public Point getEmptyPosition(Dimension sSize) { 
	Rectangle pos = new Rectangle(0, 0, sSize.width,
		sSize.height);
	Random rand = new Random(System.currentTimeMillis()); 
	boolean empty = false; 
	int numTries = 0;

	// Look for an empty position 
	while (!empty && numTries++ < 50) {
	// Get a random position 
	pos.x = Math.abs(rand.nextInt() %
	background.getSize().width); 
	pos.y = Math.abs(rand.nextInt() %
	background.getSize().height);

	// Iterate through sprites, checking if position is empty
	boolean collision = false; 
	for (int i = 0; i < size(); i++) {
		Rectangle testPos = ((Sprite)elementAt(i)).getPosition();
		if (pos.intersects(testPos)) { 
			collision = true; 
			break;
		}
		} 
		empty = !collision;
	} 
	return new Point(pos.x, pos.y);
}

Sprite isPointInside(Point pt) { 
	// Iterate backward through the sprites, testing each 
	for (int i = (size() - 1); i >= 0; i--) {
		Sprite s = (Sprite)elementAt(i); 
		if (s.isPointInside(pt))
			return s;
	}
	 return null;
}

public void update() { 
	// Iterate through sprites, updating each 
	Sprite	s, sHit; 
	Rectangle lastPos; 
	for (int i = 0; i < size(); ) {
		// Update the sprite 
		s = (Sprite)elementAt(i); 
		lastPos = new Rectangle(s.getPosition().x,
		s.getPosition().y, s.getPosition().width, s.getPosition().height);
		BitSet action = s.update();

		// Check for the SA_ADDSPRITE action 
		if (action.get(Sprite.SA_ADDSPRITE)) {
			// Add the sprite 
			Sprite sToAdd = s.addSprite(action); 
			if (sToAdd != null) {
				int iAdd = add(sToAdd); 
				if (iAdd >= 0 && iAdd <= i)
					i++;
			}
		}
		// Check for the SA_RESTOREPOS action 
		if (action.get(Sprite.SA_RESTOREPOS))
			s.setPosition(lastPos);
		// Check for the SA_KILL action 
		if (action.get(Sprite.SA_KILL)) {
			removeElementAt(i); 
			continue;
		}
		
	// Test for collision 
	int iHit = testCollision(s); 
	if (iHit >= 0)
		if (collision(i, iHit)) 
			s.setPosition(lastPos);
	i++;
	}
}


protected boolean collision(int i, int iHit) { 
	// Swap velocities (bounce) 
	Sprite s = (Sprite)elementAt(i); 
	Sprite sHit = (Sprite)elementAt(iHit); 
	Point swap = s.getVelocity(); 
	s.setVelocity(sHit.getVelocity()); 
	sHit.setVelocity(swap);
	return true;
}

protected int testCollision(Sprite test) { 
	// Check for collision with other sprites 
	Sprite s; 
	for (int i = 0; i < size(); i++)
	{
		s = (Sprite)elementAt(i); 
		if (s == test) // don't check itself
			continue; 
		if (test.testCollision(s))
			return i;
	}
	 return -1;
}

public void draw(Graphics g) { 
	// Draw the background 
	background.draw(g);

	// Iterate through sprites, drawing each 
	for (int i = 0; i < size(); i++)
		((Sprite)elementAt(i)).draw(g);
}

public int add(Sprite s) { 
	// Use a binary search to find the right location toinsert the 
	// new sprite (based on z-order)
	int l = 0, r = size(), i = 0; 
	int z = s.getZOrder(), zTest = z + 1; 
	while (r > l) {
		i = (l + r) / 2; 
		zTest = ((Sprite)elementAt(i)).getZOrder(); 
		if (z < zTest)
			r = i; 
		else
			l = i + 1; 
		if (z == zTest)
			break;
	} 
	if (z >= zTest)
		i++;

	insertElementAt(s, i); 
	return i;
}


};
